config = """{
	"keys": {
		"keep_press_mouse_left_key": "v",
		"grab_screen": "c",

		"kill": "q, q",
		"copy": "y",
		"paste": "p",

		"page_up": "ctrl+b",
		"page_down": "ctrl+f",
		"goto_top": "g, g",
		"goto_bottom": "G",

		"mouse_left_key": "space",
		"mouse_right_key": "n",
		"mouse_wheel_up": ",",
		"mouse_wheel_down": "m",

		"enter_insert_mode": "i",
		"enter_normal_mode": "ctrl+`",

		"mouse_move_down": "j",
		"mouse_move_up": "k",
		"mouse_move_left": "h",
		"mouse_move_right": "l",

		"move_window": "w",
		"direction": "r",
		"chrome": "c"
	}, 

	"mouse_move_frames": 200,
	"mouse_speed": {
		"": 10,
		"d": 30,
		"f": 2
	}, 

	"mouse_jump": {
		"b": [-400, 0],
		"e": [400, 0]
	}

}
"""

